@@ -1,15 +1,19 @@
 package WebService::Client;
 use Moo::Role;
 
-our $VERSION = '0.0601'; # VERSION
+our $VERSION = '1.0000'; # VERSION
 
 use Carp qw(croak);
 use HTTP::Request;
 use HTTP::Request::Common qw(DELETE GET POST PUT);
-use JSON::MaybeXS;
+use JSON::MaybeXS qw();
 use LWP::UserAgent;
+use WebService::Client::Response;
 
-has base_url => ( is => 'ro', required => 1 );
+has base_url => (
+    is      => 'rw',
+    default => sub { '' },
+);
 
 has ua => (
     is      => 'ro',
@@ -19,12 +23,12 @@ has ua => (
 
 has timeout => (
     is      => 'ro',
-    default => 10,
+    default => sub { 10 },
 );
 
 has retries => (
     is      => 'ro',
-    default => 0,
+    default => sub { 0 },
     isa     => sub {
         my $r = shift;
         die 'retries must be a nonnegative integer'
@@ -36,12 +40,12 @@ has logger => ( is => 'ro' );
 
 has log_method => (
     is      => 'ro',
-    default => 'DEBUG',
+    default => sub { 'DEBUG' },
 );
 
 has content_type => (
     is      => 'rw',
-    default => 'application/json',
+    default => sub { 'application/json' },
 );
 
 has deserializer => (
@@ -79,6 +83,11 @@ has json => (
     default => sub { JSON::MaybeXS->new() },
 );
 
+has mode => (
+    is      => 'ro',
+    default => sub { '' },
+);
+
 sub get {
     my ($self, $path, $params, %args) = @_;
     $params ||= {};
@@ -151,8 +160,16 @@ sub req {
         $self->_log_response($res);
     }
 
-    return if $req->method eq 'GET' and $res->code =~ /404|410/;
     $self->prepare_response($res);
+
+    if ($self->mode) {
+        return WebService::Client::Response->new(
+            res  => $res,
+            json => $self->json,
+        );
+    }
+
+    return if $req->method eq 'GET' and $res->code =~ /404|410/;
     die $res unless $res->is_success;
     return 1 unless $res->content;
     my $des = $self->deserializer;
@@ -245,7 +262,7 @@ WebService::Client - A base role for quickly and easily creating web service cli
 
 =head1 VERSION
 
-version 0.0601
+version 1.0000
 
 =head1 SYNOPSIS
 
@@ -254,12 +271,11 @@ version 0.0601
         use Moo;
         with 'WebService::Client';
 
-        use Function::Parameters;
-
-        has '+base_url' => ( default => 'https://foo.com/v1' );
         has auth_token  => ( is => 'ro', required => 1 );
 
         method BUILD() {
+            $self->base_url('https://foo.com/v1');
+
             $self->ua->default_header('X-Auth-Token' => $self->auth_token);
             # or if the web service uses http basic/digest authentication:
             # $self->ua->credentials( ... );
@@ -267,15 +283,18 @@ version 0.0601
             # $self->ua->default_headers->authorization_basic( ... );
         }
 
-        method get_widgets() {
+        sub get_widgets() {
+            my ($self) = @_;
             return $self->get("/widgets");
         }
 
-        method get_widget($id) {
+        sub get_widget($id) {
+            my ($self, $id) = @_;
             return $self->get("/widgets/$id");
         }
 
-        method create_widget($widget_data) {
+        sub create_widget($widget_data) {
+            my ($self, $widget_data) = @_;
             return $self->post("/widgets", $widget_data);
         }
     }
@@ -290,6 +309,15 @@ version 0.0601
     my $widget = $client->create_widget({ color => 'blue' });
     print $client->get_widget($widget->{id})->{color};
 
+Minimal example which retrieves the current Bitcoin price:
+
+    package CoinDeskClient;
+    use Moo;
+    with 'WebService::Client';
+
+    my $client = CoinDeskClient->new(base_url => 'https://api.coindesk.com/v1');
+    print $client->get('/bpi/currentprice.json')->{bpi}{USD}{rate_float};
+
 =head1 DESCRIPTION
 
 This module is a base role for quickly and easily creating web service clients.
